#include "types/SearchResults.h"

namespace VectorDB {

SingleResult::SingleResult(IDArray&& ids, std::vector<float>&& scores, std::vector<FieldDataPtr>&& output_fields)
    : ids_{std::move(ids)}, scores_{std::move(scores)}, output_fields_{std::move(output_fields)} {}

const std::vector<float>& SingleResult::Scores() const { return scores_; }

const IDArray& SingleResult::Ids() const { return ids_; }

const std::vector<FieldDataPtr>& SingleResult::OutputFields() const { return output_fields_; }

FieldDataPtr SingleResult::OutputField(const std::string& name) const {
    for (const auto& output_field : output_fields_) {
        if (output_field->Name() == name) {
            return output_field;
        }
    }
    return nullptr;
}

SearchResults::SearchResults() = default;

SearchResults::SearchResults(std::vector<SingleResult>&& results) { nq_results_.swap(results); }

std::vector<SingleResult>& SearchResults::Results() { return nq_results_; }

}  // namespace VectorDB
